from pathlib import Path

from PySide6.QtCore import Slot, QEvent
from PySide6.QtUiTools import loadUiType
from PySide6.QtWidgets import QDialog, QFileDialog
from vtkmodules.vtkCommonCore import VTK_INT, VTK_SHORT, VTK_LONG, VTK_FLOAT, VTK_UNSIGNED_SHORT, VTK_UNSIGNED_INT, \
    VTK_UNSIGNED_LONG

from crystalfly.thread.image_reader import ImageReaderThread

ImportFileType, _ = loadUiType(str(Path(__file__).parent.joinpath("import_file.ui")))


class ImportFile(QDialog, ImportFileType):
    def __init__(self):
        super(ImportFile, self).__init__()
        # self.image_read_thread = None
        self.setupUi(self)

        self.browse_file.clicked.connect(self.browse_file_handler)
        self.accepted.connect(self.load_image_file)

    def showEvent(self, event: QEvent) -> None:
        self.clear_form()

    def clear_form(self):
        self.file_path_input.clear()
        self.size_x_spin.clear()
        self.size_y_spin.clear()
        self.size_z_spin.clear()
        self.data_type_combobox.setCurrentIndex(0)

    @Slot()
    def load_image_file(self):
        data_type_text = self.data_type_combobox.currentText()
        data_type = None
        if data_type_text == "int 8":
            data_type = VTK_INT
        elif data_type_text == "uint 8":
            data_type = VTK_UNSIGNED_INT
        elif data_type_text == "int 16":
            data_type = VTK_SHORT
        elif data_type_text == "uint 16":
            data_type = VTK_UNSIGNED_SHORT
        elif data_type_text == "int 32":
            data_type = VTK_LONG
        elif data_type_text == "uint 32":
            data_type = VTK_UNSIGNED_LONG
        elif data_type_text == "float":
            data_type = VTK_FLOAT
        file_data = {
            "file_path": self.file_path_input.text(),
            "data_type": data_type,
            "size": {
                "x": self.size_x_spin.value(),
                "y": self.size_y_spin.value(),
                "z": self.size_z_spin.value(),
            }
        }
        image_read_thread = ImageReaderThread(file_data, self)
        # self.image_read_thread.loadedSignal.connect(self.set_image)
        image_read_thread.start()

    @Slot()
    def browse_file_handler(self):
        dialog = QFileDialog(self)
        filepath = dialog.getOpenFileName()
        if filepath[0]:
            self.file_path_input.setText(filepath[0])
