# noinspection PyUnresolvedReferences
import vtkmodules.vtkRenderingContextOpenGL2
from PySide6.QtCore import Signal
from PySide6.QtGui import QMouseEvent
from vtkmodules.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from vtkmodules.vtkCommonDataModel import vtkPiecewiseFunction
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleTrackballCamera
from vtkmodules.vtkRenderingCore import vtkVolume, vtkVolumeProperty, vtkRenderer, vtkColorTransferFunction
from vtkmodules.vtkRenderingVolume import vtkGPUVolumeRayCastMapper
# noinspection PyUnresolvedReferences
from vtkmodules.vtkRenderingVolumeOpenGL2 import vtkSmartVolumeMapper

from crystalfly.model.image_reader import image_reader_model


class VolumeViewer(QVTKRenderWindowInteractor):
    doubleClicked = Signal(int)
    clicked = Signal(int)

    def __init__(self, quadrant: int):
        super().__init__()
        self.quadrant = quadrant
        self.render_window = self.GetRenderWindow()
        self.render_window.SetMultiSamples(0)

        self.render_window.AddRenderer(vtkRenderer())
        self.renderer = self.render_window.GetRenderers().GetFirstRenderer()
        # self.iren = vtkRenderWindowInteractor()
        # self.iren.SetRenderWindow(self.render_window)

        self.volume_mapper = vtkGPUVolumeRayCastMapper()
        self.volume_mapper.SetBlendModeToComposite()
        self.volume_mapper.SetUseJittering(1)

        self.colorTF = vtkColorTransferFunction()
        self.colorTF.AddRGBPoint(-200, 0.0, 0.0, 0.0)
        self.colorTF.AddRGBPoint(110, 0.4, 0.4, 1.0)
        self.colorTF.AddRGBPoint(512, 1.0, 1.0, 1.0)

        self.scalarTF = vtkPiecewiseFunction()
        self.scalarTF.AddPoint(-255, 0.00)
        self.scalarTF.AddPoint(110, 0.00)
        self.scalarTF.AddPoint(512, 0.5)

        self.gradientTF = vtkPiecewiseFunction()
        self.gradientTF.AddPoint(-200, 0.0)

        self.volume_property = vtkVolumeProperty()
        self.volume_property.ShadeOn()
        self.volume_property.SetInterpolationTypeToLinear()
        self.volume_property.SetColor(self.colorTF)
        self.volume_property.SetScalarOpacity(self.scalarTF)
        # self.volume_property.SetGradientOpacity(self.gradientTF)

        self.volume = vtkVolume()
        self.volume.SetMapper(self.volume_mapper)
        self.volume.SetProperty(self.volume_property)

        self.renderer.ResetCamera()
        self.renderer.GetActiveCamera().Zoom(1.3)

        self.interactor_style = vtkInteractorStyleTrackballCamera()
        self.iren = self.render_window.GetInteractor()
        # self.iren.SetRenderWindow(self.render_window)
        self.iren.SetInteractorStyle(self.interactor_style)
        self.iren.Initialize()

        self.renderer.AddVolume(self.volume)
        image_reader_model.loadedSignal.connect(self.set_image)
        self.renderer.SetBackground(0.3, 0.3, 0.3)
        self.renderer.SetBackgroundAlpha(0.4)

    def set_image(self, image):
        self.volume_mapper.SetInputData(image)
        self.iren.SetRenderWindow(self.render_window)
        self.render_window.Render()
        self.renderer.ResetCamera()
        self.renderer.GetActiveCamera().Zoom(1.3)
        self.volume.Update()
        # self.volume_mapper.Render()

    def mouseDoubleClickEvent(self, evt: QMouseEvent):
        self.doubleClicked.emit(self.quadrant)

    # def mousePressEvent(self, ev: QMouseEvent):
    #     self.clicked.emit(self.quadrant)
