# noinspection PyUnresolvedReferences
import vtkmodules.vtkRenderingContextOpenGL2
from PySide6.QtCore import Signal
from PySide6.QtGui import QCloseEvent, QMouseEvent
from vtkmodules.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from vtkmodules.vtkInteractionImage import vtkImageViewer2
from vtkmodules.vtkInteractionStyle import vtkInteractorStyleImage

from crystalfly.model.image_reader import image_reader_model


class VTKImageViewer(QVTKRenderWindowInteractor):
    doubleClicked = Signal(int)
    clicked = Signal(int)
    range_changed = Signal(int, int)

    def __init__(self, quadrant: int):
        super().__init__()
        self.quadrant = quadrant
        # self.render_window = render_window
        self.render_window = self.GetRenderWindow()

        # image viewer
        # self.image_viewer = vtkResliceImageViewer()
        self.image_viewer = vtkImageViewer2()

        # 交互
        self.interactor_style = vtkInteractorStyleImage()
        self.iren = self.render_window.GetInteractor()
        self.iren.SetInteractorStyle(self.interactor_style)
        self.interactor_style.AddObserver("MouseWheelForwardEvent", self.mouse_wheel_forward_event)
        self.interactor_style.AddObserver("MouseWheelBackwardEvent", self.mouse_wheel_backward_event)
        self.iren.Initialize()

        # rep = vtkResliceCursorLineRepresentation.SafeDownCast(
        #     self.image_viewer.GetResliceCursorWidget().GetRepresentation())
        # # self.image_viewer.
        # rep.GetResliceCursorActor().GetCursorAlgorithm().SetReslicePlaneNormal(0)
        #
        # plane_widget = vtkImagePlaneWidget()
        # plane_widget.SetInteractor(self.iren)
        # plane_widget.On()
        image_reader_model.loadedSignal.connect(self.set_image)

    def set_image(self, image):
        if self.image_viewer.GetRenderWindow() is not None:
            self.image_viewer.SetRenderWindow(self.render_window)
        self.image_viewer.SetInputData(image)
        self.image_viewer.Render()
        max_slice = self.image_viewer.GetSliceMax()
        self.range_changed.emit(self.quadrant, max_slice)
        print("setImage")

    def mouse_wheel_forward_event(self, a, b):
        if self.image_viewer.GetSlice() < self.image_viewer.GetSliceMax():
            print(self.image_viewer.GetSlice())
            self.image_viewer.SetSlice(self.image_viewer.GetSlice() + 1)
            self.image_viewer.Render()

    def mouse_wheel_backward_event(self, a, b):
        if self.image_viewer.GetSlice() > self.image_viewer.GetSliceMin():
            print(self.image_viewer.GetSlice())
            self.image_viewer.SetSlice(self.image_viewer.GetSlice() - 1)
            self.image_viewer.Render()

    def closeEvent(self, evt: QCloseEvent):
        super().closeEvent(evt)
        self.Finalize()

    def mouseDoubleClickEvent(self, evt: QMouseEvent):
        self.doubleClicked.emit(self.quadrant)

    # def mousePressEvent(self, ev: QMouseEvent):
    #     self.clicked.emit(self.quadrant)
